import 'package:backend_dart/common/error_on_null.dart';
import 'package:backend_dart/domain/data/user_data_source.dart';
import 'package:backend_dart/domain/entities/user.dart';
import 'package:backend_dart/domain/errors/app_error.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/infrastructure/persistence/db/model.dart';
import 'package:backend_dart/infrastructure/persistence/db/prisma.dart';
import 'package:backend_dart/infrastructure/persistence/mapper/user_dbo_mapper.dart';
import 'package:fpdart/fpdart.dart';
import 'package:orm/orm.dart';
import 'package:uuid/uuid.dart';

import 'db/client.dart';

class PrismaUserDataSource implements UserDataSource<User> {
  final PrismaClient prisma;
  final UserDboMapper mapper = UserDboMapper();
  PrismaUserDataSource(this.prisma);

  @override
  TaskEither<IError, User> create(User user) {
    return TaskEither<IError, UserDbo>.tryCatch(
      () async {
        if (user.password == null) {
          throw Exception('Password is required');
        }
        if (user.email == null) {
          throw Exception('Email is required');
        }
        final createdUser = await prisma.userDbo.create(
          data: PrismaUnion.$1(UserDboCreateInput(
            id: user.id,
            name: user.name!,
            email: user.email!,
            password: user.password!,
          )),
        );
        return createdUser;
      },
      (error, _) => AppError.databaseError(
        message: 'Failed to create user: ${error.toString()}',
      ),
    ).flatMap(mapper.from);
  }

  @override
  TaskEither<IError, User> findByEmail(String email) {
    return TaskEither<IError, UserDbo?>.tryCatch(
      () async {
        final user = await prisma.userDbo
            .findUnique(where: UserDboWhereUniqueInput(email: email));
        if (user == null) return null;
        return user;
      },
      (error, _) => AppError.databaseError(
        message: 'Failed to find user by email: ${error.toString()}',
      ),
    )
        .flatMap(errorOnNull(AppError.notFound(
          'User with email $email found',
        )))
        .flatMap(mapper.from);
  }

  @override
  TaskEither<IError, User> findById(String id) {
    return TaskEither<IError, UserDbo?>.tryCatch(
      () async {
        final user = await prisma.userDbo
            .findUnique(where: UserDboWhereUniqueInput(id: id));
        return user;
      },
      (error, _) => AppError.databaseError(
        message: 'Failed to find user by ID: ${error.toString()}',
      ),
    )
        .flatMap(errorOnNull(AppError.notFound(
          "User with id $id not found",
        )))
        .flatMap(mapper.from);
  }

  @override
  TaskEither<IError, User> update(User user) {
    return mapper
        .to(user)
        .flatMap((userDbo) => TaskEither.tryCatch(() async {
              // Führe das Update durch
              if (userDbo.id == null) {
                throw Exception('User ID is required');
              }

              final updatedUser = await prisma.userDbo.update(
                data: PrismaUnion.$2(
                  UserDboUncheckedUpdateInput(
                    id: PrismaUnion.$1(userDbo.id!),
                    name: userDbo.name != null
                        ? PrismaUnion.$1(userDbo.name!)
                        : null,
                    email: userDbo.email != null
                        ? PrismaUnion.$1(userDbo.email!)
                        : null,
                    password: userDbo.password != null
                        ? PrismaUnion.$1(userDbo.password!)
                        : null,
                  ),
                ),
                where: UserDboWhereUniqueInput(id: userDbo.id),
              );

              return updatedUser;
            },
                (error, _) => AppError.databaseError(
                      message: 'Failed to update user: ${error.toString()}',
                    )))
        .flatMap(errorOnNull(AppError.notFound(
          'User not found',
        )))
        .flatMap(mapper.from);
  }

  @override
  TaskEither<IError, void> delete(String id) {
    return TaskEither.tryCatch(
      () async {
        await prisma.userDbo.delete(where: UserDboWhereUniqueInput(id: id));
      },
      (error, _) => AppError.databaseError(
        message: 'Failed to delete user: ${error.toString()}',
      ),
    );
  }

  @override
  TaskEither<IError, List<User>> findAll() {
    return TaskEither<IError, Iterable<UserDbo>>.tryCatch(
      () async => await prisma.userDbo.findMany(),
      (error, _) => AppError.databaseError(
        message: 'Failed to fetch all users: ${error.toString()}',
      ),
    ).flatMap(mapper.listFrom);
  }

  @override
  TaskEither<IError, String> generateId() {
    return TaskEither.tryCatch(
      () async {
        var uuid = Uuid();
        do {
          final id = uuid.v4();
          final user = await prisma.userDbo.findUnique(
            where: UserDboWhereUniqueInput(id: id),
          );
          if (user == null) return id;
        } while (true);
      },
      (error, _) => AppError.databaseError(
        message: 'Failed to generate ID: ${error.toString()}',
      ),
    );
  }
}
